<?php

namespace Drupal\Tests\handy_cache_tags\Unit;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\handy_cache_tags\HandyCacheTagsManager;
use Drupal\Tests\PhpunitCompatibilityTrait;
use Drupal\Tests\UnitTestCase;

/**
 * Test description.
 *
 * @group handy_cache_tags
 */
class TagNamesTest extends UnitTestCase {

  use PhpunitCompatibilityTrait;

  /**
   * Tests procedural with no mocking.
   */
  public function testProcedural() {
    require_once __DIR__ . '/../../../handy_cache_tags.module';
    $container = new ContainerBuilder();
    // The handler has no dependencies, so we can use it directly.
    $tag_manager = new HandyCacheTagsManager();
    $container->set('handy_cache_tags.manager', $tag_manager);
    \Drupal::setContainer($container);
    $mock_entity = $this->getMockEntity(TRUE);
    $mock_entity->expects($this->exactly(3))
      ->method('getEntityTypeId')
      ->willReturn('mock_entity');
    $mock_entity->expects($this->exactly(2))
      ->method('bundle')
      ->willReturn('mock_bundle');
    $this->assertArrayEquals([
      'handy_cache_tags:mock_entity',
      'handy_cache_tags:mock_entity:mock_bundle',
    ], handy_cache_tags_get_entity_tags($mock_entity));
    $this->assertEquals('handy_cache_tags:node', handy_cache_tags_get_tag('node'));
    $this->assertEquals('handy_cache_tags:mock_entity:mock_bundle', handy_cache_tags_get_bundle_tag_from_entity($mock_entity));
    $this->assertEquals('handy_cache_tags:node:article', handy_cache_tags_get_bundle_tag('node', 'article'));
  }

  /**
   * Test oop class name thing.
   */
  public function testHandler() {
    $tag_manager = new HandyCacheTagsManager();
    $mock_entity = $this->getMockEntity();
    $this->assertArrayEquals([
      'handy_cache_tags:mock_entity',
      'handy_cache_tags:mock_entity:mock_bundle',
    ], $tag_manager->getEntityTags($mock_entity));
    $this->assertEquals('handy_cache_tags:node', $tag_manager->getTag('node'));
    $this->assertEquals('handy_cache_tags:node:article', $tag_manager->getBundleTag('node', 'article'));
  }

  /**
   * Gets a mock entity.
   */
  private function getMockEntity($supply_expects = FALSE) {
    $mock_entity = $this->createMock('Drupal\Core\Entity\EntityInterface');
    if ($supply_expects) {
      return $mock_entity;
    }
    $mock_entity->expects($this->exactly(2))
      ->method('getEntityTypeId')
      ->willReturn('mock_entity');
    $mock_entity->expects($this->once())
      ->method('bundle')
      ->willReturn('mock_bundle');
    return $mock_entity;
  }

}
